# `kdfeatures`

This library provides functionality to investigate differential features in probability density functions.
Code is provided for reproducibility purposes as a supporting element for the manuscript *Bump hunting through density curvature features*.
See the license agreement below for further details.

## Project structure

The project comprises the following elements:

+ **License**: `LICENSE.txt`.
MIT License for this project's code.

+ **Source code**: Under `src`. 
Python source code (`*.py`) with the fundamental `kdfeatures` modules required by our research.
Dataset CSV files (`*.csv`) are included as resource files that can be accessed through Python methods.
We include the scripts used to build those datasets under `src/kdfeatures/applications/**/api.py`.
See execution instructions below.
Template images (`*.png`, `*.jpg`) for the figures also belong here.

+ **Setup files**: `setup.py` and `MANIFEST.in`.
Standard Python files to install the `kdfeatures` package.

+ **Test code**: Under `test`.
Standard Python files to perform unit tests on `kdfeatures`' functionality.

+ **Notebooks**: Under `nb`.
Jupyter notebook source files (`*.ipynb`) and their plain Python equivalent versions (`*.py`).
They contain the code used in our experiments, in particular, to generate figures.
Importantly, the notebook cells in these files will remain empty until the Python kernel executes them.
See execution instructions below.

+ **Dependency files**: Files `requirements-*.txt`.
Standard Python files for specifying package dependencies.
Requirements are split across different files depending on their use (`api` - data download, `nb` - notebooks, `src` - main `kdfeatures` package, `test` - unit tests).

+ **Container files**: `Dockerfile` and `.dockerignore`.
Standard Docker files to reproduce the execution environment for our experiments through containerization.
See execution instructions below.

## Execution instructions

The following lines instruct on how to reproduce our results and download the datasets used in the applications.

### Containerization

Our research can be entirely reproduced by building and running a container.
The only prerequisite is having [Docker](https://www.docker.com/) installed.
The same results should be achieved regardless of the hosting machine.

To build and run the Docker image, we can use, for example, the following command from the source code's root directory.

```commandline
docker build --tag kdfeatures_image .
docker run --name kdfeatures_container --rm -p 8888:8888 kdfeatures_image
```

Whichever Docker execution method is used, we remind the reader to stop and clean up all the container resources once finished.

Upon successful building, the running container will hold under its `home/nb` directory the executed versions of the Jupyter notebooks and the image files for the manuscript's figures (`home/nb/jpg/*.jpg`).
The files can be retrieved in several ways: via volumes, using a GUI Docker tool, and so on.
Also, the running container exposes a Jupyter Lab service that can be accessed through this [URL](http://localhost:8888/lab).
There, the reader can see the rendered notebooks and the final images, and execute arbitrary Python code using `kdfeatures`.

### Datasets

The application datasets are included as CSV files in the source code.
Nonetheless, data can be downloaded by calling specific APIs.
We provide some scripts with the source code that ease the download process.

#### NFL data

Use the `src/kdfeatures/applications/nfl/api.py` script with the corresponding command line arguments for each season.
Raw records are aggregated *on the fly* when reading data through the methods in the `src/kdfeatures/applications/nfl/read.py` module.

+ Tom Brady's 2014 New England Patriots:

```
--player-name
"Tom Brady"
--year
2014
--csv
tom_brady_2014.csv
```

+ Tom Brady's 2016 New England Patriots:

```
--player-name
"Tom Brady"
--year
2016
--csv
tom_brady_2016.csv
```

+ Tom Brady's 2018 New England Patriots:

```
--player-name
"Tom Brady"
--year
2018
--csv
tom_brady_2018.csv
```

#### NBA data

Use the `src/kdfeatures/applications/nba/api.py` script with the corresponding command line arguments for each player.

+ Stephen Curry:

```
--player-name
"stephen curry"
--season
2015-16
--csv
stephen_curry_2015_16.csv
```

+ James Harden:

```
--player-name
"james harden"
--season
2015-16
--csv
james_harden_2015_16.csv
```

+ Kevin Durant:

```
--player-name
"kevin durant"
--season
2015-16
--csv
kevin_durant_2015_16.csv
```

#### MLB data

Use the `src/kdfeatures/applications/mlb/api.py` script with the corresponding command line arguments for each player.

+ Clayton Kershaw:

```
--player-first-name
clayton
--player-last-name
kershaw
--start-date
2019-03-20
--end-date
2019-10-30
--csv
clayton_kershaw_2019.csv
```

+ Justin Verlander:

```
--player-first-name
justin
--player-last-name
verlander
--start-date
2019-03-20
--end-date
2019-10-30
--csv
justin_verlander_2019.csv
```

+ Max Scherzer:

```
--player-first-name
max
--player-last-name
scherzer
--start-date
2019-03-20
--end-date
2019-10-30
--csv
max_scherzer_2019.csv
```