"""
Grid building utils.
"""

from typing import Tuple

import numpy as np


def rectangular_grid(*grid_configs: Tuple[float, float, int]):
    """
    Builds a rectangular grid.

    :param grid_configs: Tuples of the form (min, max, num) specifying each
     coordinate grid.
    :return: Two objetcs: a list with all points in the rectangular grid and
     a tuple of n-dimensional arrays with the margin data disaggregated to use
     with the contour function.
    """
    linspaces = list()

    for grid_config in grid_configs:
        linspaces.append(
            np.linspace(
                start=grid_config[0], stop=grid_config[1], num=grid_config[2]
            )
        )

    linspaces = np.meshgrid(*linspaces, indexing="ij")

    return np.vstack(list(map(lambda x: x.ravel(), linspaces))).T, linspaces
