"""
Risk measures.
"""
from typing import Final

import numpy as np


def tail_value_at_risk(x: np.ndarray, p: float) -> float:
    """
    Tail Value-at-Risk (TVaR).

    :param x: Data array.
    :param p: Confidence level for the TVaR.
    :return: The TVaR.
    """
    q: Final = float(np.quantile(a=x, q=p))

    tvar: Final = q + float(np.mean(np.maximum(x - q, 0))) / (1 - p)
    assert tvar >= q

    return tvar
