"""
Tests on bootstrap methods.
"""
from typing import Final

import numpy as np
import pytest

from conftest import random_state
from kdfeatures.density import Gaussian, bootstrap_sample


@pytest.mark.parametrize(
    "dimension,sample_size,resample_size",
    [(10, 1000, 100), (15, 500, 100)],
)
def test_bootstrap_sample(dimension: int, sample_size: int, resample_size: int):
    """
    Tests the basic bootstrap resampling method.

    :param dimension: Number of components in the random vectors.
    :param sample_size: Original sample size.
    :param resample_size: Bootstrap sample size.
    """
    # Fixture sample.

    gaussian: Final = Gaussian(dimension=dimension)
    sample: Final = gaussian.sample(size=sample_size, rand=random_state)

    # Bootstrap sample from the original fixture sample.

    resample: Final = bootstrap_sample(
        sample=sample, resample_size=resample_size, rand=random_state
    )

    # Checks.

    assert sample.shape[1] == resample.shape[1] == dimension
    assert len(resample) == resample_size
    assert len(np.unique(resample, axis=0)) < len(sample)
