"""
Probability density function basic functionality testing.
"""
from typing import List, Tuple

import numpy as np
import pytest
from kdfeatures.density.gaussian import Gaussian

from kdfeatures.density import PDF


@pytest.mark.parametrize(
    "pdf,ranges",
    [
        (
            Gaussian(1, mean=np.array([0.0]), covariance=np.array([[1.0]])),
            [[-5.0, 5.0]],
        ),
        (
            Gaussian(
                2,
                mean=np.array([0.0, 1.0]),
                covariance=np.array([[1.0, 0.5], [0.5, 1.0]]),
            ),
            [[-5.0, 5.0], [-4.0, 6.0]],
        ),
        (
            Gaussian(
                3,
                mean=np.array([0.0, 1.0, -1.0]),
                covariance=np.array(
                    [[1.0, 0.5, 0.3], [0.5, 1.0, 0.7], [0.3, 0.7, 1.0]]
                ),
            ),
            [[-5.0, 5.0], [-4.0, 6.0], [-6.0, 4.0]],
        ),
    ],
)
def test_pdf_integral(pdf: PDF, ranges: List[Tuple[float, float]]):
    """
    Tests the integration functionality for a pdf, checking that the total
    probability mass in a big region is approximately equal to 1.

    :param pdf: pdf to be tested.
    :param ranges: Integration domain.
    """
    assert pdf.integral(ranges) == pytest.approx(1.0, abs=1e-4)
